package com.marsching.pustefix.eclipse.popup.actions;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.marsching.pustefix.eclipse.Plugin;
import com.marsching.pustefix.eclipse.preferences.PustefixPreferencePage;

public class GenerateIWrapperAction implements IObjectActionDelegate {
	
	private Vector files;
	
	private void reportError(Exception e, Shell shell) {
		MessageDialog.openError(shell, e.getClass().getName(), "An exception occured:\n" + e.getMessage());
	}
	
	/**
	 * Constructor for Action1.
	 */
	public GenerateIWrapperAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		
		if (this.files.isEmpty())
			return;
		
		InputStream styleStream = null;
		try {
			styleStream = Plugin.getDefault().openStream(new Path("iwrp2java.xml"));
		} catch (IOException e) {
			this.reportError(e, shell);
			return;
		}
		
		TransformerFactory tFab = TransformerFactory.newInstance();
		Transformer transformer = null;
		try {
			transformer = tFab.newTransformer(new StreamSource(styleStream));
		} catch (TransformerConfigurationException e) {
			this.reportError(e, shell);
			return;
		}
		
		Iterator i = this.files.iterator();
		while (i.hasNext()) {
			IFile file = (IFile) i.next();
			String filename = file.getName();
			filename = filename.substring(0, filename.lastIndexOf('.'));
			String classname = filename;
			filename = filename + ".java";
			IJavaElement ije = JavaCore.create(file.getParent());
			
			if (ije == null || !(ije instanceof IPackageFragment)) {
				MessageDialog.openError(shell, "IWrapper not in package", "The IWrapper file is not within a Java package but has to be for generation!");
				return;
			}
			
			IPackageFragment ipf = (IPackageFragment) ije;
			String pack = ipf.getElementName();
			
			if (pack.equals("")) {
				MessageDialog.openError(shell, "IWrapper in default package", "The IWrapper file cannot be in the default Java package!");
				return;
			}

			IFile outputfile = file.getParent().getFile(new Path(filename));
			if (outputfile.exists()) {
				if (Plugin.getDefault().getPreferenceStore().getBoolean(PustefixPreferencePage.P_PROMT_BEFORE_OVERWRITE) && !MessageDialog.openQuestion(shell, "Target file existing", "The file '" + filename + "'is already existing.\nDo you want to overwrite the existing file?"))
					break;
			} else {
				try {
					outputfile.create(new StringBufferInputStream(""), false, null);
				} catch (CoreException e) {
					this.reportError(e, shell);
					return;				
				}
			}
			try {
				InputStream iwrpStream = file.getContents();
				
				ByteArrayOutputStream javaStream = new ByteArrayOutputStream();
			
				transformer.setParameter("classname", classname);
				transformer.setParameter("package", pack);
				
				transformer.transform(new StreamSource(iwrpStream), new StreamResult(javaStream));
				
				transformer.clearParameters();

				outputfile.setContents(new ByteArrayInputStream(javaStream.toByteArray()), false, false, null);
				
				iwrpStream.close();
				javaStream.close();
			} catch (IOException e) {
				this.reportError(e, shell);
				return;
			} catch (CoreException e) {
				this.reportError(e, shell);
				return;
			} catch (TransformerException e) {
				this.reportError(e, shell);
				return;
			}
		}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection != null && !selection.isEmpty() && selection instanceof StructuredSelection) {
			StructuredSelection s = (StructuredSelection) selection;
			this.files = new Vector();
			for (Iterator i = s.iterator(); i.hasNext();) {
				IResource r = (IResource) i.next();
				if (r != null && r instanceof IFile) {
					IFile f = (IFile) r;
					if (f.getFileExtension().equals("iwrp")) {
						this.files.add(f);
					}
				}
			}
		}
	}

}
